clear j
close all;
c = ureal('c',0.5,'range',[0 1]);
G11=tf([6 6],[1 5 6]);
G12=tf([c],[1 20]);
G21=G12;
G22=tf([0.1 1],[1 2 1]);
G=[G11 G12;G21 G22];
G12nom=tf([c.Nominal],[1 20]);
Gnom=[G11 G12nom;G12nom G22];

w=logspace(-5,5,101);
Nw=length(w);

%========= Defining WP ============
wB=3; % desired closed-loop bandwidth
A=1e-3; % desired disturbance attenuation inside bandwidth
M=1.7; % desired bound on hinfnorm(S) & hinfnorm(T)
n=1;    % the slope of the weight function
WP=zpk([-wB*M^(1/n)*ones(n,1)],[-wB*A^(1/n)*ones(n,1)],1/M);
WPm=[WP 0; 0 WP]; 

%========= Defining WO ============
zz=-0.15;
pp=-50;
Wo=zpk([zz],[pp],0.03*pp/zz);
Wom=[Wo 0 ;0 Wo];

%========= Defining WS ============
ws=tf(0.5,[1 20]);
Wsm = [0 ws; ws 0];

%========= Defining P =============
P = [zeros(2,2) zeros(2,2) Wsm;
    WPm WPm WPm*Gnom;
    -eye(2) -eye(2) -Gnom];
P = minreal(ss(P));

nmeas = 2; %dimension of v
ncon = 2; %dimension of u
blk = [2 0;2 2];

%======== Step 0 =================
DL = eye(4);
DR = eye(4);
load DLRdata
DLm = [DL zeros(4,2);zeros(2,4) eye(2)];
DRm = [DR zeros(4,2);zeros(2,4) eye(2)];
P = minreal(ss(DLm*P*inv(DRm)))

%======== Step 1 =================
[K1,CL,GAM,INFO] = hinfsyn(P,nmeas,ncon,'Tolgam',1e-3);
N1 = lft(P,K1,2,2);

%======== Step 2 =================
N1f=frd(N1,w);
[mubnds,muinfo]=mussv(N1f,blk,'c');
figure(1);
bodemag(mubnds(:,1),'r',mubnds(:,2),'b');
title('\mu(N(j\omega))');
xlabel('\omega');

figure(2);
bodemag(K1);
title('The controller freq response');
xlabel('\omega');

%======== Step 3 =================
[dsysl,dsysr]=mussvunwrap(muinfo);
dlnorm = zeros(Nw,1);
for i=1:Nw
    dlnorm(i)=norm(freqresp(dsysl,w(i)));
end;
dl=fitfrd(genphase(frd(dlnorm,w)),4);

figure(3);
bodemag(dl,'b',frd(dlnorm,w),'r');
title('The fitted left weigth (blue) vs actual (red)');
xlabel('\omega');

drnorm = zeros(Nw,1);
for i=1:Nw
    drnorm(i)=norm(freqresp(dsysr,w(i)));
end;
dr=fitfrd(genphase(frd(drnorm,w)),4);
figure(4);
bodemag(dr,'b',frd(drnorm,w),'r');
title('The fitted right weigth (blue) vs actual (red)');
xlabel('\omega');

DL = eye(4)*dl;
DR = eye(4)*dr;

save DLRdata DL DR




